/* (C) 2022 YiRing, Inc. */
package com.yiring.app.push.rabbitmq;

import java.util.HashMap;
import java.util.Map;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.TopicExchange;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 推送 rabbitmq 配置
 *
 * @author ifzm
 * 2019/8/21 15:44
 */
@Configuration
public class PushRabbitConfig {

    public static final String PUSH_TOPIC_EXCHANGE = "push.topic.exchange";

    public static final String PUSH_APP_QUEUE = "push.app.queue";
    public static final String PUSH_WS_QUEUE = "push.ws.queue";
    public static final String PUSH_SMS_QUEUE = "push.sms.queue";
    public static final String PUSH_EMAIL_QUEUE = "push.email.queue";
    public static final String PUSH_CALL_TTS_QUEUE = "push.call.tts.queue";
    public static final String PUSH_CALL_AUDIO_QUEUE = "push.call.audio.queue";
    public static final String PUSH_DEVICE_TONE_QUEUE = "push.device.tone.queue";

    /**
     * 订阅模式
     *
     * @return TopicExchange
     */
    @Bean
    TopicExchange exchange() {
        return new TopicExchange(PUSH_TOPIC_EXCHANGE, true, false);
    }

    @Bean(PUSH_APP_QUEUE)
    public Queue pushAppQueue() {
        Map<String, Object> args = new HashMap<>(1);
        args.put("x-message-ttl", 60 * 1000);
        return new Queue(PUSH_APP_QUEUE, true, false, false, args);
    }

    @Bean
    Binding bindingExchangeApp(@Qualifier(PUSH_APP_QUEUE) Queue queue, TopicExchange exchange) {
        return BindingBuilder.bind(queue).to(exchange).with(PUSH_APP_QUEUE);
    }

    @Bean(PUSH_WS_QUEUE)
    public Queue pushWebSocketQueue() {
        Map<String, Object> args = new HashMap<>(1);
        args.put("x-message-ttl", 30 * 1000);
        return new Queue(PUSH_WS_QUEUE, true, false, false, args);
    }

    @Bean
    Binding bindingExchangeWs(@Qualifier(PUSH_WS_QUEUE) Queue queue, TopicExchange exchange) {
        return BindingBuilder.bind(queue).to(exchange).with(PUSH_WS_QUEUE);
    }

    @Bean(PUSH_SMS_QUEUE)
    public Queue pushSmsQueue() {
        Map<String, Object> args = new HashMap<>(1);
        args.put("x-message-ttl", 30 * 1000);
        return new Queue(PUSH_SMS_QUEUE, true, false, false, args);
    }

    @Bean
    Binding bindingExchangeSms(@Qualifier(PUSH_SMS_QUEUE) Queue queue, TopicExchange exchange) {
        return BindingBuilder.bind(queue).to(exchange).with(PUSH_SMS_QUEUE);
    }

    @Bean(PUSH_EMAIL_QUEUE)
    public Queue pushEmailQueue() {
        Map<String, Object> args = new HashMap<>(1);
        args.put("x-message-ttl", 60 * 1000);
        return new Queue(PUSH_EMAIL_QUEUE, true, false, false, args);
    }

    @Bean
    Binding bindingExchangeEmail(@Qualifier(PUSH_EMAIL_QUEUE) Queue queue, TopicExchange exchange) {
        return BindingBuilder.bind(queue).to(exchange).with(PUSH_EMAIL_QUEUE);
    }

    @Bean(PUSH_CALL_TTS_QUEUE)
    public Queue pushCallTtsQueue() {
        Map<String, Object> args = new HashMap<>(1);
        args.put("x-message-ttl", 60 * 10 * 1000);
        return new Queue(PUSH_CALL_TTS_QUEUE, true, false, false, args);
    }

    @Bean
    Binding bindingExchangeCallTts(@Qualifier(PUSH_CALL_TTS_QUEUE) Queue queue, TopicExchange exchange) {
        return BindingBuilder.bind(queue).to(exchange).with(PUSH_CALL_TTS_QUEUE);
    }

    @Bean(PUSH_CALL_AUDIO_QUEUE)
    public Queue pushCallAudioQueue() {
        Map<String, Object> args = new HashMap<>(1);
        args.put("x-message-ttl", 60 * 10 * 1000);
        return new Queue(PUSH_CALL_AUDIO_QUEUE, true, false, false, args);
    }

    @Bean
    Binding bindingExchangeCallAudio(@Qualifier(PUSH_CALL_AUDIO_QUEUE) Queue queue, TopicExchange exchange) {
        return BindingBuilder.bind(queue).to(exchange).with(PUSH_CALL_AUDIO_QUEUE);
    }

    @Bean(PUSH_DEVICE_TONE_QUEUE)
    public Queue pushDeviceToneQueue() {
        Map<String, Object> args = new HashMap<>(1);
        args.put("x-message-ttl", 60 * 10 * 1000);
        return new Queue(PUSH_DEVICE_TONE_QUEUE, true, false, false, args);
    }

    @Bean
    Binding bindingExchangeDeviceTone(@Qualifier(PUSH_DEVICE_TONE_QUEUE) Queue queue, TopicExchange exchange) {
        return BindingBuilder.bind(queue).to(exchange).with(PUSH_DEVICE_TONE_QUEUE);
    }
}
