/* (C) 2022 YiRing, Inc. */
package com.yiring.app.web;

import com.yiring.app.param.location.LocationTagAddParam;
import com.yiring.app.param.location.LocationTagDeleteParam;
import com.yiring.app.param.location.LocationTagFindParam;
import com.yiring.app.param.location.LocationTagModifyParam;
import com.yiring.app.service.location.LocationTagService;
import com.yiring.app.vo.location.LocationTagVo;
import com.yiring.common.core.Result;
import com.yiring.common.param.IdParam;
import com.yiring.common.vo.PageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * 定位标签
 * @author LJ-2204
 * @date 2022/4/13
 */

@Slf4j
@Validated
@Api(tags = "定位标签")
@RestController
@RequestMapping("/location/tag/")
public class LocationTagController {

    @Resource
    LocationTagService locationTagService;

    @ApiOperation("新增定位标签")
    @PostMapping("addLocationTag")
    public Result<String> addLocationTag(@Valid LocationTagAddParam param) {
        return locationTagService.addLocationTag(param);
    }

    @ApiOperation("分页查询")
    @GetMapping("findLocationTagPage")
    public Result<PageVo<LocationTagVo>> findLocationTagPage(@Valid LocationTagFindParam param) {
        return locationTagService.findLocationTagPage(param);
    }

    @ApiOperation("销毁定位标签")
    @DeleteMapping("deleteLocationTag")
    public Result<String> deleteLocationTag(@Valid LocationTagDeleteParam param) {
        return locationTagService.deleteLocationTag(param);
    }

    @ApiOperation("导出定位标签")
    @GetMapping("exportLocationTagInfo")
    public void exportLocationTagInfo(@Valid LocationTagFindParam param, HttpServletResponse response) {
        locationTagService.exportLocationTagInfo(param, response);
    }

    @ApiOperation("导入定位标签")
    @PostMapping("importLocationTagInfo")
    public Result<String> importLocationTagInfo(@RequestParam("file") MultipartFile file) {
        return locationTagService.importLocationTagInfo(file);
    }

    @ApiOperation("修改定位标签")
    @PutMapping("modifyLocationTag")
    public Result<String> modifyLocationTag(@Valid LocationTagModifyParam param, IdParam idParam) {
        return locationTagService.modifyLocationTag(param, idParam);
    }
}
