/* (C) 2022 YiRing, Inc. */
package com.yiring.auth.util;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.http.Header;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.yiring.common.core.Result;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import lombok.experimental.UtilityClass;
import org.springframework.data.redis.core.RedisTemplate;

/**
 * 真源授权工具
 * @author LJ-2204
 * @date 2022/4/15
 */

@UtilityClass
public class ZyUtil {

    RedisTemplate<String, Object> redisTemplate;

    static {
        redisTemplate = SpringUtil.getBean("redisTemplate");
    }

    public final String CLIENT_ID = "sc21080400";

    public final String CLIENT_SECRET = "12A14FDC";

    public final String GRANT_TYPE = "client_credentials";

    public final String KEY = "zy_token";

    public final String TOKEN_TYPE = "bearer ";

    public final String URL_PREFIX = "http://project.yz-online.com:789";

    /**
     * 获取token
     * @return token
     */
    public static String login() {
        Object obj = redisTemplate.opsForValue().get(KEY);
        if (ObjectUtil.isNotEmpty(obj)) {
            return String.valueOf(obj);
        }
        Map<String, Object> map = MapUtil.createMap(HashMap.class);
        map.put("client_id", CLIENT_ID);
        map.put("client_secret", CLIENT_SECRET);
        map.put("grant_type", GRANT_TYPE);
        String json = HttpUtil.get("http://project.yz-online.com:789/positionApi/oauth/token", map);
        Map result = JSON.parseObject(json, Map.class);
        String access_token = result.get("access_token").toString();
        redisTemplate.opsForValue().set(KEY, access_token, 1200, TimeUnit.SECONDS);
        return access_token;
    }

    /**
     * 判断token是否有效
     * @return T/F
     */
    public static boolean getToken() {
        return ObjectUtil.isNotEmpty(redisTemplate.opsForValue().get(KEY));
    }

    /**
     * get请求
     * @param urlString 路径
     * @param obj 参数
     * @param timeout 请求时长
     * @return json
     */
    public static Result get(String urlString, Object obj, int timeout) {
        String json = JSONUtil.toJsonStr(obj);
        String token = login();
        String body = HttpRequest
            .get(URL_PREFIX + urlString)
            .header(Header.AUTHORIZATION, TOKEN_TYPE + token)
            .header(Header.CONTENT_TYPE, "application/json")
            .body(json)
            .timeout(timeout)
            .execute()
            .body();
        return getResult(body);
    }

    /**
     * get请求
     * @param urlString 路径
     * @param obj 路径参
     * @param timeout 请求时长
     * @return json
     */
    public static Result<Serializable> getRequest(String urlString, Object obj, int timeout) {
        String json = JSONUtil.toJsonStr(obj);
        String token = login();
        String body = HttpRequest
            .get(URL_PREFIX + urlString + json)
            .header(Header.AUTHORIZATION, TOKEN_TYPE + token)
            .header(Header.CONTENT_TYPE, "application/json")
            //            .body(json)
            .timeout(timeout)
            .execute()
            .body();
        return bodyResult(body);
    }

    /**
     * post请求
     * @param urlString 路径
     * @param obj 参数
     * @param timeout 请求时长
     * @return json
     */
    public static Result<Serializable> post(String urlString, Object obj, int timeout) {
        String json = JSONUtil.toJsonStr(obj);
        String token = login();
        String body = HttpRequest
            .post(URL_PREFIX + urlString)
            .header(Header.AUTHORIZATION, TOKEN_TYPE + token)
            .header(Header.CONTENT_TYPE, "application/json")
            .body(json)
            .timeout(timeout)
            .execute()
            .body();
        return getResult(body);
    }

    /**
     * put请求
     * @param urlString 路径
     * @param obj 参数
     * @param timeout 请求时长
     * @return json
     */
    public static Result<Serializable> put(String urlString, Object obj, int timeout) {
        String json = JSONUtil.toJsonStr(obj);
        String token = login();
        String body = HttpRequest
            .put(URL_PREFIX + urlString)
            .header(Header.AUTHORIZATION, TOKEN_TYPE + token)
            .header(Header.CONTENT_TYPE, "application/json")
            .body(json)
            .timeout(timeout)
            .execute()
            .body();
        return getResult(body);
    }

    /**
     * delete请求
     * @param urlString 路径
     * @param obj 参数
     * @param timeout 请求时长
     * @return json
     */
    public static Result<Serializable> delete(String urlString, Object obj, int timeout) {
        String json = JSONUtil.toJsonStr(obj);
        String token = login();
        String body = HttpRequest
            .delete(URL_PREFIX + urlString)
            .header(Header.AUTHORIZATION, TOKEN_TYPE + token)
            .header(Header.CONTENT_TYPE, "application/json")
            .body(json)
            .timeout(timeout)
            .execute()
            .body();
        return getResult(body);
    }

    /**
     * 返回类型转换
     * @param body json
     * @return Result
     */
    private static Result<Serializable> getResult(String body) {
        Map map = JSONUtil.toBean(body, Map.class);
        Integer code = Convert.toInt(map.get("code"));
        String msg = Convert.toStr(map.get("msg"));
        String bodyResult = Convert.toStr(map.get("data"));
        return Result.builder().code(code).message(msg).body(bodyResult).build();
    }

    /**
     * 返回类型转换
     * @param body json
     * @return Result
     */
    private static Result<Serializable> bodyResult(String body) {
        Map map = JSONUtil.toBean(body, Map.class);
        Integer code = Convert.toInt(map.get("code"));
        String msg = Convert.toStr(map.get("msg"));
        String data = Convert.toStr(map.get("data"));
        return Result.builder().code(code).message(msg).body(data).build();
    }
}
